@@ -23,6 +23,7 @@
 
 #include <assert.h>
 #include <iostream>
+#include "gsl/gsl_cblas.h"
 #include "gsl/gsl_matrix.h"
 
 gsl_matrix *fast_copy(gsl_matrix *m, const double *mem);
@@ -37,4 +38,6 @@ void fast_eigen_dgemm(const char *TransA, const char *TransB, const double alpha
                       const gsl_matrix *A, const gsl_matrix *B, const double beta,
                       gsl_matrix *C);
 
+void fast_inverse(gsl_matrix *m);
+
 #endif
