@@ -522,10 +522,11 @@ double LogL_f(double l, void *params) {
   index_yy = GetabIndex(n_cvt + 2, n_cvt + 2, n_cvt);
   double P_yy = gsl_matrix_safe_get(Pab, nc_total, index_yy);
 
+  if (P_yy == 0.0) P_yy = 0.00000001; // control potential round-off
   if (is_check_mode() || is_debug_mode()) {
     // cerr << "P_yy is" << P_yy << endl;
     assert(!is_nan(P_yy));
-    assert(P_yy > 0);
+    assert(P_yy > 0.0);
   }
   f = c - 0.5 * logdet_h - 0.5 * (double)ni_test * safe_log(P_yy);
   if (is_check_mode() || is_debug_mode()) {
@@ -846,6 +847,8 @@ double LogRL_f(double l, void *params) {
   }
   index_ww = GetabIndex(n_cvt + 2, n_cvt + 2, n_cvt);
   double P_yy = gsl_matrix_safe_get(Pab, nc_total, index_ww);
+  // P_yy is positive and may get zeroed printf("P_yy=%f",P_yy);
+  if (P_yy == 0.0) P_yy = 0.00000001; // control potential round-off
 
   double c = 0.5 * df * (safe_log(df) - safe_log(2 * M_PI) - 1.0);
   f = c - 0.5 * logdet_h - 0.5 * logdet_hiw - 0.5 * df * safe_log(P_yy);
@@ -1422,7 +1425,7 @@ void LMM::AnalyzeGene(const gsl_matrix *U, const gsl_vector *eval,
     CalcUab(UtW, Uty, Uab);
     FUNC_PARAM param0 = {false, ni_test, n_cvt, eval, Uab, ab, 0};
 
-    if (a_mode == M_LMM2 || a_mode == M_LMM3 || a_mode == M_LMM4) {
+    if (a_mode == M_LMM2 || a_mode == M_LMM3 || a_mode == M_LMM4 || a_mode == M_LMM9) {
       CalcLambda('L', param0, l_min, l_max, n_region, l_H0, logl_H0);
     }
 
@@ -1431,7 +1434,7 @@ void LMM::AnalyzeGene(const gsl_matrix *U, const gsl_vector *eval,
     FUNC_PARAM param1 = {false, ni_test, n_cvt, eval, Uab, ab, 0};
 
     // 3 is before 1.
-    if (a_mode == M_LMM3 || a_mode == M_LMM4) {
+    if (a_mode == M_LMM3 || a_mode == M_LMM4 || a_mode == M_LMM9) {
       CalcRLScore(l_H0, param1, beta, se, p_score);
     }
 
@@ -1440,7 +1443,7 @@ void LMM::AnalyzeGene(const gsl_matrix *U, const gsl_vector *eval,
       CalcRLWald(lambda_remle, param1, beta, se, p_wald);
     }
 
-    if (a_mode == M_LMM2 || a_mode == M_LMM4) {
+    if (a_mode == M_LMM2 || a_mode == M_LMM4 || a_mode == M_LMM9) {
       CalcLambda('L', param1, l_min, l_max, n_region, lambda_mle, logl_H1);
       p_lrt = gsl_cdf_chisq_Q(2.0 * (logl_H1 - logl_H0), 1);
     }
@@ -1540,7 +1543,7 @@ void LMM::Analyze(std::function< SnpNameValues(size_t) >& fetch_snp,
         CalcRLWald(lambda_remle, param1, beta, se, p_wald);
       }
 
-      if (a_mode == M_LMM2 || a_mode == M_LMM9 || a_mode == M_LMM4) {
+      if (a_mode == M_LMM2 || a_mode == M_LMM4 || a_mode == M_LMM9) {
         CalcLambda('L', param1, l_min, l_max, n_region, lambda_mle, logl_H1);
         p_lrt = gsl_cdf_chisq_Q(2.0 * (logl_H1 - logl_mle_H0), 1);
       }
@@ -1852,17 +1855,18 @@ void LMM::AnalyzePlink(const gsl_matrix *U, const gsl_vector *eval,
         FUNC_PARAM param1 = {false, ni_test, n_cvt, eval, Uab, ab, 0};
 
         // 3 is before 1, for beta.
-        if (a_mode == M_LMM3 || a_mode == M_LMM4) {
+        if (a_mode == M_LMM3 || a_mode == M_LMM4 || a_mode == M_LMM9) {
           CalcRLScore(l_mle_null, param1, beta, se, p_score);
         }
 
         if (a_mode == M_LMM1 || a_mode == M_LMM4) {
           CalcLambda('R', param1, l_min, l_max, n_region, lambda_remle,
                      logl_H1);
-          CalcRLWald(lambda_remle, param1, beta, se, p_wald);
+          if (!isnan(logl_H1))
+            CalcRLWald(lambda_remle, param1, beta, se, p_wald);
         }
 
-        if (a_mode == M_LMM2 || a_mode == M_LMM4) {
+        if (a_mode == M_LMM2 || a_mode == M_LMM4 || a_mode == M_LMM9) {
           CalcLambda('L', param1, l_min, l_max, n_region, lambda_mle, logl_H1);
           p_lrt = gsl_cdf_chisq_Q(2.0 * (logl_H1 - logl_mle_H0), 1);
         }
@@ -1870,6 +1874,9 @@ void LMM::AnalyzePlink(const gsl_matrix *U, const gsl_vector *eval,
         time_opt += (clock() - time_start) / (double(CLOCKS_PER_SEC) * 60.0);
 
         // Store summary data.
+        if (isnan(logl_H1)) { // invalidate values
+          p_wald = p_lrt = logl_H1;
+        }
         SUMSTAT SNPs = {beta,   se,    lambda_remle, lambda_mle,
                         p_wald, p_lrt, p_score, logl_H1};
         sumStat.push_back(SNPs);
@@ -1933,6 +1940,10 @@ void MatrixCalcLR(const gsl_matrix *U, const gsl_matrix *UtX,
 void CalcLambda(const char func_name, FUNC_PARAM &params, const double l_min,
                 const double l_max, const size_t n_region, double &lambda,
                 double &logf) {
+  // wipe return values
+  logf = nan("NAN");
+  lambda = nan("NAN");
+
   if (func_name != 'R' && func_name != 'L' && func_name != 'r' &&
       func_name != 'l') {
     cout << "func_name only takes 'R' or 'L': 'R' for "
@@ -1943,6 +1954,7 @@ void CalcLambda(const char func_name, FUNC_PARAM &params, const double l_min,
   vector<pair<double, double>> lambda_lh;
 
   // Evaluate first-order derivates in different intervals.
+  assert(l_max > l_min);
   double lambda_l, lambda_h,
       lambda_interval = safe_log(l_max / l_min) / (double)n_region;
   double dev1_l, dev1_h, logf_l, logf_h;
@@ -1984,9 +1996,7 @@ void CalcLambda(const char func_name, FUNC_PARAM &params, const double l_min,
   } else {
 
     // If derivates change signs.
-    int status;
-    int iter = 0, max_iter = 100;
-    double l, l_temp;
+    double l=0.0, l_temp = 0.0;
 
     gsl_function F;
     gsl_function_fdf FDF;
@@ -2006,42 +2016,76 @@ void CalcLambda(const char func_name, FUNC_PARAM &params, const double l_min,
       FDF.fdf = &LogL_dev12;
     }
 
-    const gsl_root_fsolver_type *T_f;
-    gsl_root_fsolver *s_f;
-    T_f = gsl_root_fsolver_brent;
-    s_f = gsl_root_fsolver_alloc(T_f);
-
-    const gsl_root_fdfsolver_type *T_fdf;
-    gsl_root_fdfsolver *s_fdf;
-    T_fdf = gsl_root_fdfsolver_newton;
-    s_fdf = gsl_root_fdfsolver_alloc(T_fdf);
+    const gsl_root_fsolver_type *T_f = gsl_root_fsolver_brent;
+    const gsl_root_fsolver *s_f = gsl_root_fsolver_alloc(T_f);
+    const gsl_root_fdfsolver_type *T_fdf = gsl_root_fdfsolver_newton;
+    const gsl_root_fdfsolver *s_fdf = gsl_root_fdfsolver_alloc(T_fdf);
 
     for (vector<double>::size_type i = 0; i < lambda_lh.size(); ++i) {
       lambda_l = lambda_lh[i].first;
       lambda_h = lambda_lh[i].second;
-      gsl_root_fsolver_set(s_f, &F, lambda_l, lambda_h);
+      gsl_root_fsolver_set((gsl_root_fsolver*)s_f, &F, lambda_l, lambda_h);
 
+      int status = GSL_FAILURE;
+      uint iter = 0;
+      const auto max_iter = 100;
+
+      auto handler = gsl_set_error_handler_off();
       do {
         iter++;
-        status = gsl_root_fsolver_iterate(s_f);
+        status = gsl_root_fsolver_iterate((gsl_root_fsolver*)s_f);
+        if (status != GSL_SUCCESS && status != GSL_CONTINUE) {
+          warning_msg("Brent did not converge");
+          break;
+        }
         l = gsl_root_fsolver_root(s_f);
         lambda_l = gsl_root_fsolver_x_lower(s_f);
         lambda_h = gsl_root_fsolver_x_upper(s_f);
         status = gsl_root_test_interval(lambda_l, lambda_h, 0, 1e-1);
+        if (status != GSL_SUCCESS && status != GSL_CONTINUE) {
+          debug_msg("Brent did not converge");
+          break;
+        }
       } while (status == GSL_CONTINUE && iter < max_iter);
 
-      iter = 0;
-
-      gsl_root_fdfsolver_set(s_fdf, &FDF, l);
+      if (status == GSL_CONTINUE) {
+        debug_msg("Brent root did not converge: too many iterations");
+        break;
+      }
 
+      uint iter2 = 0;
+      gsl_root_fdfsolver_set((gsl_root_fdfsolver*)s_fdf, &FDF, l);
       do {
-        iter++;
-        status = gsl_root_fdfsolver_iterate(s_fdf);
+        iter2++;
+        status = gsl_root_fdfsolver_iterate((gsl_root_fdfsolver*)s_fdf);
+        if (status != GSL_SUCCESS && status != GSL_CONTINUE) {
+          debug_msg("Newton did not converge");
+          break;
+        }
         l_temp = l;
-        l = gsl_root_fdfsolver_root(s_fdf);
+        l = gsl_root_fdfsolver_root((gsl_root_fdfsolver*)s_fdf);
         status = gsl_root_test_delta(l, l_temp, 0, 1e-5);
-      } while (status == GSL_CONTINUE && iter < max_iter && l > l_min &&
-               l < l_max);
+        if (status != GSL_SUCCESS && status != GSL_CONTINUE) {
+          debug_msg("Newton did not converge");
+          break;
+        }
+      } while (status == GSL_CONTINUE && iter2 < max_iter && l > l_min && l < l_max);
+
+      // cleanup
+      gsl_set_error_handler(handler);
+
+      if (status == GSL_CONTINUE) {
+        debug_msg("Newton root did not converge: too many iterations");
+      }
+
+      if (status == GSL_CONTINUE || status != GSL_SUCCESS) {
+        // make sure results are invalid
+        logf = nan("NAN");
+        lambda = nan("NAN");
+        gsl_root_fsolver_free((gsl_root_fsolver*)s_f);
+        gsl_root_fdfsolver_free((gsl_root_fdfsolver*)s_fdf);
+        return;
+      }
 
       l = l_temp;
       if (l < l_min) {
@@ -2065,8 +2109,8 @@ void CalcLambda(const char func_name, FUNC_PARAM &params, const double l_min,
       } else {
       }
     }
-    gsl_root_fsolver_free(s_f);
-    gsl_root_fdfsolver_free(s_fdf);
+    gsl_root_fsolver_free((gsl_root_fsolver*)s_f);
+    gsl_root_fdfsolver_free((gsl_root_fdfsolver*)s_fdf);
 
     if (func_name == 'R' || func_name == 'r') {
       logf_l = LogRL_f(l_min, &params);
@@ -2338,7 +2382,7 @@ void LMM::AnalyzeBimbamGXE(const gsl_matrix *U, const gsl_vector *eval,
     FUNC_PARAM param1 = {false, ni_test, n_cvt + 2, eval, Uab, ab, 0};
 
     // 3 is before 1.
-    if (a_mode == 3 || a_mode == 4) {
+    if (a_mode == 3 || a_mode == 4 || a_mode == 9) {
       CalcRLScore(l_mle_null, param1, beta, se, p_score);
     }
 
@@ -2347,7 +2391,7 @@ void LMM::AnalyzeBimbamGXE(const gsl_matrix *U, const gsl_vector *eval,
       CalcRLWald(lambda_remle, param1, beta, se, p_wald);
     }
 
-    if (a_mode == 2 || a_mode == 4) {
+    if (a_mode == 2 || a_mode == 4 || a_mode == 9) {
       CalcLambda('L', param1, l_min, l_max, n_region, lambda_mle, logl_H1);
       p_lrt = gsl_cdf_chisq_Q(2.0 * (logl_H1 - logl_H0), 1);
     }
@@ -2516,7 +2560,7 @@ void LMM::AnalyzePlinkGXE(const gsl_matrix *U, const gsl_vector *eval,
     FUNC_PARAM param1 = {false, ni_test, n_cvt + 2, eval, Uab, ab, 0};
 
     // 3 is before 1, for beta.
-    if (a_mode == 3 || a_mode == 4) {
+    if (a_mode == 3 || a_mode == 4 || a_mode == 9) {
       CalcRLScore(l_mle_null, param1, beta, se, p_score);
     }
 
@@ -2525,7 +2569,7 @@ void LMM::AnalyzePlinkGXE(const gsl_matrix *U, const gsl_vector *eval,
       CalcRLWald(lambda_remle, param1, beta, se, p_wald);
     }
 
-    if (a_mode == 2 || a_mode == 4) {
+    if (a_mode == 2 || a_mode == 4 || a_mode == 9) {
       CalcLambda('L', param1, l_min, l_max, n_region, lambda_mle, logl_H1);
       p_lrt = gsl_cdf_chisq_Q(2.0 * (logl_H1 - logl_H0), 1);
     }
