@@ -150,11 +150,19 @@ void CenterMatrix(gsl_matrix *G) {
   gsl_vector *Gw = gsl_vector_safe_alloc(G->size1);
   gsl_vector_set_all(w, 1.0);
 
+  //  y := alpha*A*x+ beta*y or Gw = G*w
   gsl_blas_dgemv(CblasNoTrans, 1.0, G, w, 0.0, Gw);
+
+  // int gsl_blas_dsyr2(CBLAS_UPLO_t Uplo, double alpha, const gsl_vector * x, const gsl_vector * y, gsl_matrix * A)
+  // compute the symmetric rank-2 update A = \alpha x y^T + \alpha y x^T + A of the symmetric matrix A. Since the matrix A is symmetric only its upper half or lower half need to be stored
+  // or G = (UpperTriangle) alpha*Gw*w' + alpha*w*Gw' + G
   gsl_blas_dsyr2(CblasUpper, -1.0 / (double)G->size1, Gw, w, G);
+  // compute dot product of vectors w.Gw and store in d
   gsl_blas_ddot(w, Gw, &d);
+  // G = (upper) alpha w*w' + G
   gsl_blas_dsyr(CblasUpper, d / ((double)G->size1 * (double)G->size1), w, G);
 
+  // Transpose the matrix
   for (size_t i = 0; i < G->size1; ++i) {
     for (size_t j = 0; j < i; ++j) {
       d = gsl_matrix_get(G, j, i);
@@ -169,6 +177,7 @@ void CenterMatrix(gsl_matrix *G) {
 }
 
 // Center the matrix G.
+// Only used in vc
 void CenterMatrix(gsl_matrix *G, const gsl_vector *w) {
   double d, wtw;
   gsl_vector *Gw = gsl_vector_safe_alloc(G->size1);
@@ -192,6 +201,7 @@ void CenterMatrix(gsl_matrix *G, const gsl_vector *w) {
 }
 
 // Center the matrix G.
+// Only used in vc
 void CenterMatrix(gsl_matrix *G, const gsl_matrix *W) {
   gsl_matrix *WtW = gsl_matrix_safe_alloc(W->size2, W->size2);
   gsl_matrix *WtWi = gsl_matrix_safe_alloc(W->size2, W->size2);
@@ -233,6 +243,7 @@ void CenterMatrix(gsl_matrix *G, const gsl_matrix *W) {
 }
 
 // "Standardize" the matrix G such that all diagonal elements = 1.
+// (only used by vc)
 void StandardizeMatrix(gsl_matrix *G) {
   double d = 0.0;
   vector<double> vec_d;
@@ -260,11 +271,13 @@ void StandardizeMatrix(gsl_matrix *G) {
 double ScaleMatrix(gsl_matrix *G) {
   double d = 0.0;
 
+  // Compute mean of diagonal
   for (size_t i = 0; i < G->size1; ++i) {
     d += gsl_matrix_get(G, i, i);
   }
   d /= (double)G->size1;
 
+  // Scale the matrix using the diagonal mean
   if (d != 0) {
     gsl_matrix_scale(G, 1.0 / d);
   }
@@ -387,7 +400,11 @@ uint count_abs_small_values(const gsl_vector *v, double min) {
 // and the ratio of max and min but one (min is expected to be zero).
 bool isMatrixIllConditioned(const gsl_vector *eigenvalues, double max_ratio) {
   auto t = abs_minmax(eigenvalues);
+
+#if !defined NDEBUG
   auto absmin = get<0>(t);
+#endif
+
   auto absmin1 = get<1>(t);
   auto absmax = get<2>(t);
   if (absmax/absmin1 > max_ratio) {
