@@ -1,7 +1,9 @@
-#!/usr/bin/env bash
+#!/bin/sh
 #
 # Long running tests go here
 
+echo "WARNING: THIS TEST SUITE IS NO LONGER USED"
+
 gemma=../bin/gemma
 export GSL_RNG_SEED=100
 
